@@ -9,15 +9,107 @@ use Exporter;
           get_reg get_current_time get_max_byte
           get_exec_count get_exec_time get_last_error get_cmp_flag );
 
-$VERSION   = 0.3;
+$VERSION   = 0.4;
 
 use strict;
 use warnings;
-use Data::Dumper;
 
-##############################################################################
+use constant 
+{
+    MIX_OK              => 0,
+    MIX_HALT            => 1,
+    MIX_ERROR           => 2,
+    MIX_IOWAIT          => 3
+};
+
+# Unit 0 to 7 is tapes; Unit 8 to 15 is disks and drums.
+# To specify a tape unit, the convention here is to write (U_TAPE + i).
+# Typewriter and Papertape share the same unit number, 
+# because they are physically combined.
+use constant
+{
+    U_TAPE              => 0,
+    U_DISK              => 8,
+    U_CARDREADER        => 16,
+    U_CARDPUNCH         => 17,
+    U_PRINTER           => 18,
+    U_TYPEWRITER        => 19,
+    U_PAPERTAPE         => 19
+};
+
+# Op Dispatch Table
+my @op_dispatch_table = (
+    \&X_NOP,             #00
+    \&X_ADD,             #01
+    \&X_SUB,             #02
+    \&X_MUL,             #03
+    \&X_DIV,             #04
+    \&X_MISC,            #05
+    \&X_SHIFT,           #06
+    \&X_MOVE,            #07
+    \&X_LDA,             #08
+    \&X_LDI,             #09
+    \&X_LDI,             #10
+    \&X_LDI,             #11
+    \&X_LDI,             #12
+    \&X_LDI,             #13
+    \&X_LDI,             #14
+    \&X_LDX,             #15
+    \&X_LDAN,            #16
+    \&X_LDIN,            #17
+    \&X_LDIN,            #18
+    \&X_LDIN,            #19
+    \&X_LDIN,            #20
+    \&X_LDIN,            #21
+    \&X_LDIN,            #22
+    \&X_LDXN,            #23
+    \&X_STA,             #24
+    \&X_STI,             #25
+    \&X_STI,             #26
+    \&X_STI,             #27
+    \&X_STI,             #28
+    \&X_STI,             #29
+    \&X_STI,             #30
+    \&X_STX,             #31
+    \&X_STJ,             #32
+    \&X_STZ,             #33
+    \&X_JBUS,            #34
+    \&X_IOC,             #35
+    \&X_INPUT,           #36
+    \&X_OUTPUT,          #37
+    \&X_JRED,            #38
+    \&X_JMP_COND,        #39
+    \&X_JMP_REG,         #40
+    \&X_JMP_REG,         #41
+    \&X_JMP_REG,         #42
+    \&X_JMP_REG,         #43
+    \&X_JMP_REG,         #44
+    \&X_JMP_REG,         #45
+    \&X_JMP_REG,         #46
+    \&X_JMP_REG,         #47
+    \&X_ADDR_TRANSFER,   #48
+    \&X_ADDR_TRANSFER,   #49
+    \&X_ADDR_TRANSFER,   #50
+    \&X_ADDR_TRANSFER,   #51
+    \&X_ADDR_TRANSFER,   #52
+    \&X_ADDR_TRANSFER,   #53
+    \&X_ADDR_TRANSFER,   #54
+    \&X_ADDR_TRANSFER,   #55
+    \&X_CMP,             #56
+    \&X_CMP,             #57
+    \&X_CMP,             #58
+    \&X_CMP,             #59
+    \&X_CMP,             #60
+    \&X_CMP,             #61
+    \&X_CMP,             #62
+    \&X_CMP,             #63
+    );
+
+my @regname = qw(rA rI1 rI2 rI3 rI4 rI5 rI6 rX);
+
+################################################################
 # Initialization
-##############################################################################
+################################################################
 
 sub new 
 {
@@ -28,9 +120,6 @@ sub new
 
     $self->{max_byte} = 64 if !exists $self->{max_byte};
 
-    # According to knuth, in 1960s, one time unit on a high-priced machine is 1 us. 
-    # and a low cost is 10 us.  One time unit is same to the memory access time
-    # we want to set it to 5 us.
     $self->{timeunit} = 5  if !exists $self->{timeunit};
     $self->{ms} = 1000/$self->{timeunit};
 
@@ -80,7 +169,7 @@ sub reset
     $self->{next_pc}   = 0;
     $self->{ov_flag}   = 0;
     $self->{cmp_flag}  = 0;
-    $self->{status}    = 0;
+    $self->{status}    = MIX_OK;
     $self->{message}   = 'running';
 }
 
@@ -88,16 +177,162 @@ sub reset
 # Instruction Execution
 ##############################################################################
 
+# Execute an instruction, update the machine state
+# Return 1 if successfully execute one; 0 if there are something happen.
+# On returning zero, it is not necessarily an error, maybe
+# you just need to type a line to get the program continue.
+sub step
+{
+    my $self = shift;
+
+    if ($self->{status} == MIX_IOWAIT) { # Need to read a line to proceed
+	return 0;
+    } elsif ($self->{status} != MIX_OK) {
+	return 0;
+    }
+
+    my $start_time = $self->{time};
+
+    # Fetch instruction
+
+    my $loc = $self->{pc};
+    if ($loc < 0 || $loc > 3999)
+    {
+	$self->{status} = MIX_ERROR;
+	$self->{message} = "instruction overflow: $loc";
+	return 0;
+    }
+
+    # one time unit for one memory access
+    $self->{time} = $self->{time} + 1;
+
+    # read instruction and unpack the fields
+    my @word = @{@{$self->{mem}}[$loc]};
+    my $c = $word[5];
+    my $f = $word[4];
+    my $r = $f % 8;
+    my $l = int($f / 8);
+    my $i = $word[3];
+    my $a = $word[1] * $self->{max_byte} + $word[2];
+    $a = ($word[0] eq '+')? $a : (0 - $a);
+    my $m = $a;
+    if ($i >= 1 && $i <= 6)
+    {
+	my $ireg = $self->{$regname[$i]};
+	my $offset = @{$ireg}[4] * $self->{max_byte} + @{$ireg}[5];
+	$offset = 0 - $offset if (@{$ireg}[0] eq '-');
+	$m += $offset;
+    }
+    
+    # default next program counter
+    $self->{next_pc} = $self->{pc} + 1;
+
+    # execute instruction
+    if ($c >= 0 || $c <= 63 )
+    {
+	my $opfunc = $op_dispatch_table[$c];
+	goto ERROR_INST if &{$opfunc}($self, $c, $f, $r, $l, $i, $a, $m) == 0;
+    }
+    else
+    {
+ERROR_INST:
+	$self->{status} = MIX_ERROR;
+	$self->{message} = "invalid instruction at $loc";
+	return 0;
+    }
+
+    # update program counter
+    $self->{pc} = $self->{next_pc};
+
+    # update performance data
+    @{$self->{execnt}}[$loc]++;
+    @{$self->{exetime}}[$loc] += $self->{time} - $start_time;
+
+    return 1;
+}
+
+# MIX "GO" button
+sub go 
+{
+    my ($self) = @_;
+
+    $self->load_card(0);
+    while ($self->{status} == MIX_OK) {
+	$self->step();
+    }
+}
+
+sub X_ADD {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    $self->add(\@tmp);
+    return 1;
+}
+
+sub X_ADDR_TRANSFER {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    my $reg = $self->{$regname[$c-48]};
+    if ($f == 0) { ## INC
+	my $v = word_to_int($reg, $self->{max_byte});
+	if (int_to_word($v+$m, $reg, $self->{max_byte})) {
+	    $self->{ov_flag} = 0;
+	} else {
+	    $self->{ov_flag} = 1;
+	}
+    } elsif ($f == 1) { ## DEC
+	my $v = word_to_int($reg, $self->{max_byte});
+	if (int_to_word($v-$m, $reg, $self->{max_byte})) {
+	    $self->{ov_flag} = 0;
+	} else {
+	    $self->{ov_flag} = 1;
+	}
+    } elsif ($f == 2) { ##ENT
+	int_to_word($m, $reg, $self->{max_byte});
+    } elsif ($f == 3) { ##ENN
+	int_to_word(-$m, $reg, $self->{max_byte});
+    } else {
+	return 0;
+    }
+
+    return 1;
+}
+
+sub X_CMP {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    my $tmp1 = $self->get_reg($regname[$c-56], $l, $r);
+    my $tmp2 = $self->read_mem_timed($m, $l, $r);
+    $self->{cmp_flag} = $tmp1 - $tmp2;
+
+    return 1;
+}
+
+sub X_DIV {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    return 0 if $f == 6;
+
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    $self->div(\@tmp);
+
+    # DIV requires 10 additional time units
+    $self->{time} += 10;
+    return 1;
+}
+
 # Usage: $self->wait_until_device_ready($devnum)
 #
 # Used only before IN/OUT operations. 
 # 
 # If the device is busy, that is, the current time - last started < delay,
 # increase the current time, so that the device would be ready
-
 sub wait_until_device_ready
 {
     my ($self, $devnum) = @_;
+
+    return if $devnum < 0 || $devnum > 19;
+
     my $devstat = @{$self->{devstat}}[$devnum];
     my $laststarted = $devstat->{laststarted};
 
@@ -110,216 +345,158 @@ sub wait_until_device_ready
     }
 }
 
-# Execute an instruction, update the machine state
-sub step
-{
-    my $self = shift;
+sub X_INPUT {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
 
-    my @regname = qw(rA rI1 rI2 rI3 rI4 rI5 rI6 rX);
-
-    return if $self->{status} != 0;
-
-    my $start_time = $self->{time};
-
-    # Fetch instruction
-    my $loc = $self->{pc};
-    my @word = $self->read_mem_timed($loc);
-    return if $self->{status} != 0;
-
-    my $c = $word[5];
-    my $f = $word[4];
-    my $r = $f%8;
-    my $l = int($f/8);
-    my $i = $word[3];
-    my $a = $word[1] * $self->{max_byte} + $word[2];
-    $a = ($word[0] eq '+')? $a : (0 - $a);
-    my $m = $a;
-    if ($i >= 1 && $i <= 6) {
-	$m += $self->get_reg('rI' . $i);
+    $self->wait_until_device_ready($f);
+    if ($f == U_CARDREADER) { ## CARD READER
+	$self->load_card($m);
+    } elsif ($f >= U_TAPE && $f < U_DISK) {
+	$self->read_tape($f, $m);
+    } elsif ($f >= U_DISK && $f < U_CARDREADER) {
+	$self->read_disk($f, $m);
+    } elsif ($f == U_TYPEWRITER) { # Input from typewriter
+	$self->read_typewriter($m);
+    } else {
+	$self->{status} = MIX_ERROR;
+	$self->{message} = "invalid input device(#$f)";
     }
-    
-    $self->{next_pc} = $self->{pc} + 1;
+    return 1;
+}
 
-    if ( $c == 5 && $f == 2) { ## HLT: the machine stops
-	$self->{status} = 1;
-	$self->{message} = 'halts normally';
-    } elsif ($c == 0) { ## NOP: no operation
-    } elsif ($c == 8) { ## LDA: load A
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	$self->set_reg('rA', \@tmp);
-    } elsif ($c == 15) { ## LDX
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	$self->set_reg('rX', \@tmp);
-    } elsif ($c >= 9 && $c <= 14) { ## LDi
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	$self->set_reg('rI' . ($c-8), \@tmp);
-    } elsif ($c == 16) { ## LDAN
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	@tmp = neg_word(\@tmp);
-	$self->set_reg('rA', \@tmp);
-    } elsif ($c == 23) { ## LDXN
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	@tmp = neg_word(\@tmp);
-	$self->set_reg('rX', \@tmp);
-    } elsif ($c >= 17 && $c <= 22) { ## LDiN
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	@tmp = neg_word(\@tmp);
-	$self->set_reg('rI' . ($c-16), \@tmp);
-    } elsif ($c == 24) { ## STA
-	$self->write_mem_timed($m, $self->{rA}, $l, $r);
-    } elsif (25 <= $c && $c <= 30) { ## STi
-	my $ri = 'rI' . ($c-24);
-	$self->write_mem_timed($m, $self->{$ri}, $l, $r);
-    } elsif ($c == 31) { ## STX
-	$self->write_mem_timed($m, $self->{rX}, $l, $r);
-    } elsif ($c == 32) { ## STJ
-	$self->write_mem_timed($m, $self->{rJ}, $l, $r);
-    } elsif ($c == 33) { ## STZ
-	$self->write_mem_timed($m, $self->{rZ}, $l, $r);
-    } elsif ($c == 1) { ## ADD
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	$self->add(\@tmp);
-    } elsif ($c == 2) { ## SUB
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	$self->minus(\@tmp);
-    } elsif ($c == 3 && $f != 6) { ## MUL
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	$self->mul(\@tmp);
+sub X_IOC {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
 
-        # MUL requires 8 additional time units
-        $self->{time} += 8;
-    } elsif ($c == 4 && $f != 6) { ## DIV
-	my @tmp = $self->read_mem_timed($m, $l, $r);
-	$self->div(\@tmp);
+    $self->wait_until_device_ready($f);
+    if ($f == U_PRINTER) { ## Printer: set up new page
+	$self->new_page($m);
+    } elsif (U_TAPE <= $f && $f <= (U_TAPE+7)) {
+	$self->set_tape_pos($f, $m);
+    } elsif (U_DISK <= $f && $f <= (U_DISK+7)) {
+	$self->set_disk_pos($f);
+    } elsif ($f == U_PAPERTAPE) {
+	$self->rewind_paper_tape;
+    } else {
+	$self->{status} = MIX_ERROR;
+	$self->{message} = "invalid ioc for device(#$f)";
+    }
+    return 1;
+}
 
-        # DIV requires 10 additional time units
-        $self->{time} += 10;
-    } elsif (48 <= $c && $c <= 55) { 
-	my $reg = $self->{$regname[$c-48]};
-	if ($f == 0) { ## INC
-	    my $v = word_to_int($reg, $self->{max_byte});
-	    if (int_to_word($v+$m, $reg, $self->{max_byte})) {
-		$self->{ov_flag} = 0;
-	    } else {
-		$self->{ov_flag} = 1;
-	    }
-	} elsif ($f == 1) { ## DEC
-	    my $v = word_to_int($reg, $self->{max_byte});
-	    if (int_to_word($v-$m, $reg, $self->{max_byte})) {
-		$self->{ov_flag} = 0;
-	    } else {
-		$self->{ov_flag} = 1;
-	    }
-	} elsif ($f == 2) { ##ENT
-	    int_to_word($m, $reg, $self->{max_byte});
-	} elsif ($f == 3) { ##ENN
-	    int_to_word(-$m, $reg, $self->{max_byte});
-	} else {
-	    goto  ERROR_INST;
-	}
-    } elsif (56 <= $c && $c <= 63) { ## CMP
-	my $tmp1 = $self->get_reg($regname[$c-56], $l, $r);
-	my $tmp2 = $self->read_mem_timed($m, $l, $r);
-	$self->{cmp_flag} = $tmp1 - $tmp2;
-    } elsif ($c == 39) { ## JMP ON CONDITION
-	goto ERROR_INST if $f > 9;
-	my $ok   = 1;
-	my $savj = 0;
-	my $cf   = $self->{cmp_flag};
-	my @cond = ($cf<0,$cf==0,$cf>0,$cf>=0,$cf!=0,$cf<=0);
+# Jump when device busy: always no busy
+sub X_JBUS {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    return 1;
+}
 
-	if ($f == 0) {
-	    $ok = 1;
-	}elsif ($f == 1) {
-	    $savj = 1;
-	} elsif ($f == 2) {
-	    $ok = $self->{ov_flag};
-	} elsif ($f == 3) {
-	    $ok = !$self->{ov_flag};
-	} else {
-	    $ok = $cond[$f-4];
-	}
+sub X_JMP_COND {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
 
-	if ($ok) {
-	    if (!$savj) {
-		int_to_word($self->{next_pc}, $self->{rJ}, $self->{max_byte});
-	    }
-	    $self->{next_pc} = $m;
-	}
-    } elsif (40 <= $c && $c <= 47) {
-	goto ERROR_INST if $f > 5;
-	my $val = $self->get_reg($regname[$c-40]);
-	my @cond = ($val<0,$val==0,$val>0,$val>=0,$val!=0,$val<=0);
-	if ($cond[$f]) {
+    return 0 if $f > 9;
+    my $ok   = 1;
+    my $savj = 0;
+    my $cf   = $self->{cmp_flag};
+    my @cond = ($cf<0,$cf==0,$cf>0,$cf>=0,$cf!=0,$cf<=0);
+
+    if ($f == 0) {
+	$ok = 1;
+    }elsif ($f == 1) {
+	$savj = 1;
+    } elsif ($f == 2) {
+	$ok = $self->{ov_flag};
+    } elsif ($f == 3) {
+	$ok = !$self->{ov_flag};
+    } else {
+	$ok = $cond[$f-4];
+    }
+
+    if ($ok) {
+	if (!$savj) {
 	    int_to_word($self->{next_pc}, $self->{rJ}, $self->{max_byte});
-	    $self->{next_pc} = $m;
 	}
-    } elsif ($c == 7) {
-	my $dest = $self->get_reg('rI1');
-	for (my $i = 0; $i < $f; $i++, $m++, $dest++) {
-	    my @w = $self->read_mem_timed($m);
-	    $self->write_mem_timed($dest, \@w);
-	}
-	my @tmp = ('+', 0,0,0,0,0);
-	int_to_word($dest, \@tmp, $self->{max_byte});
-	$self->set_reg('rI1', \@tmp);
-    } elsif ($c == 6) { ## Shift Operators
-	goto ERROR_INST if $m < 0;
+	$self->{next_pc} = $m;
+    }
 
-	my @a = @{$self->{rA}};
-	my @x = @{$self->{rX}};
-	my $sa = shift @a;
-	my $sx = shift @x;
-	if ($f == 0) { ## SLA
-	    $m = $m%5;
-	    while (--$m >= 0) {
-		shift @a;
-		push @a, 0;
-	    }
-	} elsif ($f == 1) { ## SRA
-	    $m = $m%5;
-	    while (--$m >= 0) {
-		pop @a;
-		unshift @a, 0;
-	    }
-	} elsif ($f == 2) { ## SLAX
-	    $m = $m%10;
-	    while (--$m >= 0) {
-		shift @a;
-		push @a, shift @x;
-		push @x, 0;
-	    }
-	} elsif ($f == 3) { ## SRAX
-	    $m = $m%10;
-	    while (--$m >= 0) {
-		pop @x;
-		unshift @x, pop @a;
-		unshift @a, 0;
-	    }
-	} elsif ($f == 4) { ## SLC
-	    $m = $m%10;
-	    while (--$m >= 0) {
-		push @a, shift @x;
-		push @x, shift @a;
-	    }
-	} elsif ($f == 5) { ## SRC
-	    $m = $m%10;
-	    while (--$m >= 0) {
-		unshift @a, pop @x;
-		unshift @x, pop @a;
-	    }
-	} else {
-	    goto ERROR_INST;
-	}
-	unshift @a, $sa;
-	unshift @x, $sx;
-	$self->set_reg('rA', \@a);
-	$self->set_reg('rX', \@x);
+    return 1;
+}
 
-        # shift operations takes additional 1 time unit
-        $self->{time} = $self->{time} + 1;
+sub X_JMP_REG {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    return 0 if $f > 5;
+    my $val = $self->get_reg($regname[$c-40]);
+    my @cond = ($val<0,$val==0,$val>0,$val>=0,$val!=0,$val<=0);
+    if ($cond[$f]) {
+	int_to_word($self->{next_pc}, $self->{rJ}, $self->{max_byte});
+	$self->{next_pc} = $m;
+    }
+    return 1;
+}
 
-    } elsif ($c == 5 && $f == 0) { ## NUM
+# Jump ready: jump immediately
+sub X_JRED {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    int_to_word($self->{next_pc}, $self->{rJ}, $self->{max_byte});
+    $self->{next_pc} = $m;
+
+    return 1;
+}
+
+sub X_LDA {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    $self->set_reg('rA', \@tmp);
+    return 1;
+}
+
+sub X_LDAN {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    @tmp = neg_word(\@tmp);
+    $self->set_reg('rA', \@tmp);
+    return 1;
+}
+
+sub X_LDI {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    $self->set_reg('rI' . ($c-8), \@tmp);
+    return 1;
+}
+
+sub X_LDIN {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    @tmp = neg_word(\@tmp);
+    $self->set_reg('rI' . ($c-16), \@tmp);
+    return 1;
+}
+
+sub X_LDX {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    $self->set_reg('rX', \@tmp);
+    return 1;
+}
+
+sub X_LDXN {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    @tmp = neg_word(\@tmp);
+    $self->set_reg('rX', \@tmp);
+    return 1;
+}
+
+sub X_MISC {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    if ($f == 2) # HLT
+    {
+	$self->{status} = MIX_HALT;
+	$self->{message} = 'halts normally';	
+    }
+    elsif ($f == 0) # NUM
+    {
 	my @a = @{$self->{rA}};
 	my @x = @{$self->{rX}};
 	my $m = $self->{max_byte};
@@ -340,7 +517,9 @@ sub step
 	}
 	int_to_word($val, $self->{rA}, $m);
 	@{$self->{rA}}[0] = $sa;
-    } elsif ($c == 5 && $f == 1) { ## CHAR
+    }
+    elsif ($f == 1) # CHAR
+    {
 	my $val = word_to_uint($self->{rA}, $self->{max_byte});
 	my $i;
 	for ($i = 5; $i >= 1; $i--) {
@@ -351,72 +530,163 @@ sub step
 	    @{$self->{rA}}[$i] = 30 + $val%10;
 	    $val = int($val/10);
 	}
-    } elsif ($c == 36) { ## IN
-	if ($f == 16) { ## CARD READER
-            $self->wait_until_device_ready($f);
-	    $self->load_card($m);
-	} elsif ($f >= 0 && $f <= 7) {
-            $self->wait_until_device_ready($f);
-	    $self->read_tape($f, $m);
-	} elsif ($f >= 8 && $f <= 15) {
-            $self->wait_until_device_ready($f);
-	    $self->read_disk($f, $m);
-	} else {
-	    $self->{status} = 2;
-	    $self->{message} = "input device(#$f) not supported at $loc";
-	}
-    } elsif ($c == 37) {
-	if ($f == 17) { ## CARD Punch
-	    $self->wait_until_device_ready($f);
-            $self->punch_card($m);
-	} elsif ($f == 18)  { ## Printer
-	    $self->wait_until_device_ready($f);
-	    $self->print_line($m);
-	} elsif (0 <= $f && $f <= 7) {
-	    $self->wait_until_device_ready($f);
-	    $self->write_tape($f, $m);
-	} elsif (8 <= $f && $f <= 15) {
-	    $self->wait_until_device_ready($f);
-	    $self->write_disk($f, $m);
-	} else {
-	    $self->{status} = 2;
-	    $self->{message} = "output device(#$f) not supported at $loc";
-	}
-    } elsif ($c == 35) {
-	if ($f == 18) { ## Printer: set up new page
-	    $self->wait_until_device_ready($f);
-	    $self->new_page($m);
-	} elsif (0 <= $f && $f <= 7) {
-	    $self->set_tape_pos($f, $m);
-	} elsif (8 <= $f && $f <= 15) {
-	    $self->set_disk_pos($f);
-	} else {
-	    $self->{status} = 2;
-	    $self->{message} = "ioctrl for device(#$f) not supported at $loc";
-	}
-    } elsif ($c == 34) { ## JBUS: Always no busy
-    } elsif ($c == 38) { ## JRED: Jump immediately		
-	int_to_word($self->{next_pc}, $self->{rJ}, $self->{max_byte});
-	$self->{next_pc} = $m;
-    } else {
-ERROR_INST:
-	$self->{status} = 2;
-	$self->{message} = "invalid instruction at $loc";
     }
-
-    @{$self->{execnt}}[$loc]++;
-    @{$self->{exetime}}[$loc] += $self->{time} - $start_time;
-    $self->{pc} = $self->{next_pc};
+    return 1;
 }
 
-sub go 
-{
-    my ($self) = @_;
+sub X_MOVE {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
 
-    $self->load_card(0);
-    while ($self->{status} == 0) {
-	$self->step();
+    my $dest = $self->get_reg('rI1');
+    for (my $i = 0; $i < $f; $i++, $m++, $dest++) {
+	my @w = $self->read_mem_timed($m);
+	$self->write_mem_timed($dest, \@w);
     }
+    my @tmp = ('+', 0,0,0,0,0);
+    int_to_word($dest, \@tmp, $self->{max_byte});
+    $self->set_reg('rI1', \@tmp);
+
+    return 1;
+}
+
+sub X_MUL {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    return 0 if $f == 6;
+
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    $self->mul(\@tmp);
+
+    # MUL requires 8 additional time units
+    $self->{time} += 8;
+    return 1;
+}
+
+sub X_NOP {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    return 1;
+}
+
+sub X_OUTPUT {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    $self->wait_until_device_ready($f);
+    if ($f == U_CARDPUNCH) { ## CARD Punch
+	$self->punch_card($m);
+    } elsif ($f == U_PRINTER)  { ## Printer
+	$self->print_line($m);
+    } elsif (U_TAPE <= $f && $f <= (U_TAPE+7)) {
+	$self->write_tape($f, $m);
+    } elsif (U_DISK <= $f && $f <= (U_DISK+7)) {
+	$self->write_disk($f, $m);
+    } elsif ($f == U_PAPERTAPE) { ## Output to paper tape
+	$self->write_paper_tape($m);
+    } else {
+	$self->{status} = MIX_ERROR;
+	$self->{message} = "invalid output device(#$f)";
+    }
+
+    return 1;
+}
+
+sub X_SHIFT {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+
+    return 0 if $m < 0;
+
+    my @a = @{$self->{rA}};
+    my @x = @{$self->{rX}};
+    my $sa = shift @a;
+    my $sx = shift @x;
+    if ($f == 0) { ## SLA
+	$m = $m%5;
+	while (--$m >= 0) {
+	    shift @a;
+	    push @a, 0;
+	}
+    } elsif ($f == 1) { ## SRA
+	$m = $m%5;
+	while (--$m >= 0) {
+	    pop @a;
+	    unshift @a, 0;
+	}
+    } elsif ($f == 2) { ## SLAX
+	$m = $m%10;
+	while (--$m >= 0) {
+	    shift @a;
+	    push @a, shift @x;
+	    push @x, 0;
+	}
+    } elsif ($f == 3) { ## SRAX
+	$m = $m%10;
+	while (--$m >= 0) {
+	    pop @x;
+	    unshift @x, pop @a;
+	    unshift @a, 0;
+	}
+    } elsif ($f == 4) { ## SLC
+	$m = $m%10;
+	while (--$m >= 0) {
+	    push @a, shift @x;
+	    push @x, shift @a;
+	}
+    } elsif ($f == 5) { ## SRC
+	$m = $m%10;
+	while (--$m >= 0) {
+	    unshift @a, pop @x;
+	    unshift @x, pop @a;
+	}
+    } else {
+	return 0;
+    }
+
+    unshift @a, $sa;
+    unshift @x, $sx;
+    $self->set_reg('rA', \@a);
+    $self->set_reg('rX', \@x);
+
+    # shift operations takes additional 1 time unit
+    $self->{time} = $self->{time} + 1;
+
+    return 1;
+}
+
+sub X_STA {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    $self->write_mem_timed($m, $self->{rA}, $l, $r);
+    return 1;
+}
+
+sub X_STI {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my $ri = 'rI' . ($c-24);
+    $self->write_mem_timed($m, $self->{$ri}, $l, $r);
+    return 1;
+}
+
+sub X_STJ {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    $self->write_mem_timed($m, $self->{rJ}, $l, $r);
+    return 1;
+}
+
+sub X_STX {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    $self->write_mem_timed($m, $self->{rX}, $l, $r);
+    return 1;
+}
+
+sub X_STZ {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    $self->write_mem_timed($m, $self->{rZ}, $l, $r);
+    return 1;
+}
+
+sub X_SUB {
+    my ($self, $c, $f, $r, $l, $i, $a, $m) = @_;
+    my @tmp = $self->read_mem_timed($m, $l, $r);
+    $self->minus(\@tmp);
+    return 1;
 }
 
 ##############################################################################
@@ -428,27 +698,48 @@ sub get_reg
     my ($self, $reg, $l, $r) = @_;
 
     if (!exists $self->{$reg}) {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "accessing non-existed reg: $reg";
 	return undef;
     }
 
-    if (defined $l) {
+    my @retval = @{$self->{$reg}};
+    if (defined $l)
+    {
 	$r = $l if !defined $r;
-    } else {
-	$l = 0;
-	$r = 5;
+
+	if ($l == 0)
+	{
+	    $l = 1;
+	}
+	else
+	{
+	    $retval[0] = '+';
+	}
+
+	my $i = 5;
+	for (my $i = 5; $i > 0; $i--, $r--)
+	{
+	    if ($r >= $l)
+	    {
+		$retval[$i] = $retval[$r];
+	    }
+	    else
+	    {
+		$retval[$i] = 0;
+	    }
+	}
     }
 
-    my @word = @{$self->{$reg}};
-    my @retval = ();
-
-    for ($l .. $r) {
-	push @retval, $word[$_]
+    if (wantarray)
+    {
+	return @retval;
+    }
+    else
+    {
+	my $value = word_to_int(\@retval, $self->{max_byte});
+	return $value;
     }
-    @retval = fix_word(@retval);
-    my $value = word_to_int(\@retval, $self->{max_byte});
-    return wantarray? @retval : $value;
 }
 
 sub set_reg
@@ -456,7 +747,7 @@ sub set_reg
     my ($self, $reg, $wref) = @_;
 
     if (!exists $self->{$reg}) {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "accessing non-existed reg: $reg";
 	return;
     }
@@ -497,28 +788,49 @@ sub read_mem
     my ($self,$loc,$l, $r) = @_;
 
     if ($loc < 0 || $loc > 3999) {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "access invalid memory location: $loc";
 	return;
     }
 
-    my @word = @{@{$self->{mem}}[$loc]};
-    if (defined $l) 
+
+    my @retval = @{@{$self->{mem}}[$loc]};
+    if (defined $l)
     {
 	$r = $l if !defined $r;
-    }
-    else {
-	$l = 0;
-	$r = 5;
+
+	if ($l == 0)
+	{
+	    $l = 1;
+	}
+	else
+	{
+	    $retval[0] = '+';
+	}
+
+	my $i = 5;
+	for (my $i = 5; $i > 0; $i--, $r--)
+	{
+	    if ($r >= $l)
+	    {
+		$retval[$i] = $retval[$r];
+	    }
+	    else
+	    {
+		$retval[$i] = 0;
+	    }
+	}
     }
 
-    my @retval = ();
-    for ($l .. $r) {
-	push @retval, $word[$_]
-	}
-    @retval = fix_word(@retval);
-    my $value = word_to_int(\@retval, $self->{max_byte});
-    return wantarray? @retval : $value;
+    if (wantarray)
+    {
+	return @retval;
+    }
+    else
+    {
+	my $value = word_to_int(\@retval, $self->{max_byte});
+	return $value;
+    }
 }
 
 # $loc: location, must be in [0..3999]
@@ -529,13 +841,12 @@ sub write_mem
     my ($self,$loc,$wref, $l, $r) = @_;
 
     if ($loc < 0 || $loc > 3999) {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "access invalid memory location: $loc";
 	return;
     }
 
     my @word = @{$wref};
-    debug("write mem ", word_to_string(@word) );
 
     if (!defined $l) {
 	$l = 0;
@@ -544,7 +855,7 @@ sub write_mem
 	$r = $l;
     }
     my $dest = @{$self->{mem}}[$loc];
-    debug("   to loc#$loc ", word_to_string(@{$dest}), "($l:$r)");
+
     for (my $i = $r; $i >= $l;  $i--) {
 	@{$dest}[$i] = pop @word if $i > 0;
 	if ($i == 0) {
@@ -555,7 +866,7 @@ sub write_mem
 	    }
 	}
     }
-    debug("  => ", word_to_string(@{$dest}));
+
 }
 
 #######################################################################
@@ -660,9 +971,12 @@ sub get_max_byte
 sub get_last_error
 {
     my $self = shift;
-    return ""     if ($self->{status} == 0);
-    return "HALT" if ($self->{status} == 1);
-    return "ERROR: " . uc("$self->{message}") if $self->{status} >= 2;
+    my $status = $self->{status};
+    my $msg = uc($self->{message});
+    return "OK"   if $status == MIX_OK;
+    return "HALT" if $status == MIX_HALT;
+    return $msg   if $status == MIX_ERROR;
+    return "WAIT IO READY - " . $msg if $status == MIX_IOWAIT;
 }
 
 
@@ -674,6 +988,7 @@ sub get_last_error
 sub add_device 
 {
     my ($self, $u, $buf) = @_; 
+    # FIXME: paper tape?
     return 0 if $u > 19 || $u < 0;
     $self->{dev}->{$u} = {};
     if (defined $buf) {
@@ -737,6 +1052,9 @@ sub get_device_buffer
 sub write_tape 
 {
     my ($self, $u, $m) = @_;
+
+    #FIXME: error checking
+
     my $tape = $self->{dev}->{$u};
     my $n = @{$tape->{buf}};    
     for (my $i = 0; $i < 100; $i++) {	
@@ -754,6 +1072,9 @@ sub write_tape
 sub read_tape 
 {
     my ($self, $u, $m) = @_;
+
+    #FIXME: error checking
+
     my $tape = $self->{dev}->{$u};
     my $n = @{$tape->{buf}};    
 
@@ -773,13 +1094,16 @@ sub read_tape
 # seek : 10ms
 
 sub set_tape_pos {
-
+    my ($self) = @_;
 }
 sub set_disk_pos {
+    my ($self) = @_;
 }
 sub write_disk {
+    my ($self) = @_;
 }
 sub read_disk {
+    my ($self) = @_;
 }
 
 # Load cards into memory started at $loc
@@ -788,19 +1112,19 @@ sub load_card
     my ($self,$loc) = @_;
 
     # Check if card reader installed
-    if (!exists $self->{dev}->{16}) {
-	$self->{status} = 2;
+    if (!exists $self->{dev}->{+U_CARDREADER}) {
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "missing card reader";
 	return 0;
     }
 
-    my $reader = $self->{dev}->{16};
+    my $reader = $self->{dev}->{+U_CARDREADER};
     my $buf = $reader->{buf};
     my $pos = $reader->{pos};
 
     # Check if there are cards unread
     if ($pos >= @{$buf}) {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "no card in card reader";
 	return 0;
     }
@@ -816,7 +1140,7 @@ sub load_card
     for (my $i = 0; $i < 80; $i++) {
 	my $c = mix_char_code( substr($crd,$i,1) );
 	if ($c == -1) {
-	    $self->{status} = 2;
+	    $self->{status} = MIX_ERROR;
 	    $self->{message} = "invalid card: '$crd'";
 	    return 0;
 	} else {
@@ -828,9 +1152,10 @@ sub load_card
 	}
     }
 
-    my $devstat = @{$self->{devstat}}[17];
+    my $devstat = @{$self->{devstat}}[U_CARDREADER];
     $devstat->{laststarted} = $self->{time};
-    $devstat->{delay} = 100 * $self->{ms}; # Read 10 cards per second
+    # Read one card need 0.1 second
+    $devstat->{delay} = 100 * $self->{ms};
     
     return 1;
 }
@@ -840,8 +1165,8 @@ sub punch_card
 {
     my ($self, $loc) = @_;
 
-    if (!exists $self->{dev}->{17}) {
-	$self->{status} = 2;
+    if (!exists $self->{dev}->{+U_CARDPUNCH}) {
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "missing card punch";
 	return;
     }    
@@ -860,10 +1185,10 @@ sub punch_card
 	}
     }
 
-    my $dev = $self->{dev}->{17};
+    my $dev = $self->{dev}->{+U_CARDPUNCH};
     push @{$dev->{buf}}, $crd;
 
-    my $devstat = @{$self->{devstat}}[17];
+    my $devstat = @{$self->{devstat}}[U_CARDPUNCH];
     $devstat->{laststarted} = $self->{time};
     $devstat->{delay} = 500 * $self->{ms}; # Punch 2 cards per second
 }
@@ -871,9 +1196,9 @@ sub punch_card
 sub print_line 
 {
     my ($self, $loc) = @_;
-    my $printer = $self->{dev}->{18};
+    my $printer = $self->{dev}->{+U_PRINTER};
     if (!defined $printer) {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "missing printer";
 	return;
     }
@@ -898,7 +1223,7 @@ sub print_line
     $page .= $line . "\n";
     push @{$printer->{buf}}, $page;
 
-    my $devstat = @{$self->{devstat}}[18];
+    my $devstat = @{$self->{devstat}}[U_PRINTER];
     $devstat->{laststarted} = $self->{time};
     $devstat->{delay} = 100 * $self->{ms}; # Print 10 lines per second
 }
@@ -906,10 +1231,10 @@ sub print_line
 sub new_page 
 {
     my ($self, $m) = @_;
-    my $printer = $self->{dev}->{18};
+    my $printer = $self->{dev}->{+U_PRINTER};
 
     if (!defined $printer) {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "missing printer";
 	return;
     }
@@ -917,19 +1242,67 @@ sub new_page
     if ($m == 0) {
 	push @{$printer->{buf}}, "";
     } else {
-	$self->{status} = 2;
+	$self->{status} = MIX_ERROR;
 	$self->{message} = "printer ioctrl error: M should be zero";
     }
 
-    my $devstat = @{$self->{devstat}}[18];
+    my $devstat = @{$self->{devstat}}[U_PRINTER];
     $devstat->{laststarted} = $self->{time};
     $devstat->{delay} = 10 * $self->{ms};
 }
 
+
+sub read_typewriter
+{
+    my ($self, $loc) = @_;
+    # FIXME: use constant
+    my $typewriter = $self->{dev}->{19};
+
+    if (!defined $typewriter) {
+	$self->{status} = MIX_ERROR;
+	$self->{message} = "missing typewriter";
+	return 0;
+    }
+
+    if (!exists($typewriter->{line}))
+    {
+	$self->{status} = MIX_IOWAIT;
+	$self->{message} = "need to type a line";
+	return 0;
+    }
+
+    my $line = $typewriter->{line};
+    # Pad spaces to make the line has 70 characters
+    if (length($line)!=70) {
+	$line .= " " x (70-length($line));
+    }
+    my @w = ('+');
+    for (my $i = 0; $i < 70; $i++) {
+	my $c = mix_char_code( substr($line,$i,1) );
+	if ($c == -1) {
+	    $self->{status} = MIX_ERROR;
+	    $self->{message} = "invalid line: '$line'";
+	    return 0;
+	} else {
+	    push @w, $c;
+	    if (@w == 6) {
+		$self->write_mem($loc++, \@w);
+		@w = ('+');
+	    }
+	}
+    }
+
+    my $devstat = @{$self->{devstat}}[19];
+    $devstat->{laststarted} = $self->{time};
+    $devstat->{delay} = 100 * $self->{ms}; # Read 10 cards per second
+    
+    
+}
+
 sub is_halted 
 {
     my $self = shift;
-    return 0 if $self->{status} == 0;
+    return 0 if $self->{status} == MIX_OK;
     return 1;
 }
 
@@ -938,26 +1311,9 @@ sub is_halted
 # Utilities
 ########################################################################
 
-sub get_dev_name 
-{
-    my ($unit) = @_;
-    if ($unit >= 0 && $unit <= 7) {
-	return "Tape $unit";
-    } elsif ($unit >= 8 && $unit <= 15) {
-	return "Disk/Drum $unit";
-    } elsif ($unit == 16) {
-	return "Card reader";
-    } elsif ($unit == 17) {
-	return "Card punch";
-    } elsif ($unit == 18) {
-	return "Printer";
-    } elsif ($unit == 19) {
-	return "Typewriter and paper tape";
-    } else {
-	return "Null Device";
-    }
-}
-
+# Input: partial word, for example, (+ 10 20)
+# Output: add 0s to fix the word: (+ 0 0 0 10 20)
+# For load instructions when reading only part of the fields.
 sub fix_word
 {
     my @tmp = @_;
@@ -1041,24 +1397,8 @@ sub int_to_word
     return $val==0;
 }
 
-sub word_to_string
-{
-    my $retstr = '';
-    my $prefix = '';
-    foreach (@_) {
-	if ($_ eq '-') {
-	    $prefix = '-';
-	} elsif ($_ eq '+') {
-	    $prefix = '+';
-	} else {
-	    $retstr .= ($_ < 10? ' 0' : ' ') . $_;
-	}
-    }
-    return $prefix . $retstr;
-}
-
-
 my $debug_mode = 0;
+
 sub debug 
 {
     return if !$debug_mode;
@@ -1093,7 +1433,7 @@ __END__
 
 =head1 NAME
 
-Hardware::Simulator::MIX - Knuth's famous virtual machine
+Hardware::Simulator::MIX - A simulator of Knuth's famous virtual machine
 
 =head1 SYNOPSIS
   
@@ -1124,7 +1464,15 @@ The following options correspond to attribute methods described below:
     KEY                     DEFAULT
     -----------             --------------------
     max_byte                64
-    timeunit                5   (microseconds)
+    timeunit                5   (microseconds, not milliseconds)
+
+C<max_byte> is the maximum value a MIX byte can express.
+C<timeunit> is one memory access time.
+According to knuth, in 1960s, one time unit on a high-priced machine is 1 us. 
+and for a low cost computer it is 10 us.  
+To change the default settings, do like this:
+
+    $mix = Hardware::Simulator::MIX->new(max_byte => 100, timeunit => 10);
 
 =head1 MACHINE STATE
 
@@ -1136,7 +1484,9 @@ Accessing registers:
 
     $mix->{reg_name}
 
-It is a reference to a MIX word.
+It is a reference to a MIX word. A MIX word is an array of six elements.
+Element 0 is the sign of word,
+with value either '+' or '-'. Elements 1~5 are MIX bytes with numerical values.
 Available registers are listed below:
 
     REGNAME                FORMAT
@@ -1150,7 +1500,6 @@ Available registers are listed below:
     rI4                    [?, 0, 0, 0, ?, ?]
     rI5                    [?, 0, 0, 0, ?, ?]
     rI6                    [?, 0, 0, 0, ?, ?]
-    rI6                    [?, 0, 0, 0, ?, ?]
     rJ                     [?, 0, 0, 0, ?, ?]
     pc                     Integer in 0..3999
 
@@ -1159,8 +1508,13 @@ Note: the names are case sensitive.
     $mix->get_reg($reg_name);
     $mix->set_reg($reg_name, $wref);
 
+For registers rI1 ~ rI6, rJ, the bytes 1~3 are always set to zero.
+You should not modify them. Or the result will be undefined.
+
 =item Memory
 
+A MIX memory word is similar to the register rA and rX word.
+
     $mix->read_mem($loc);
     $mix->read_mem($loc, $l);
     $mix->read_mem($loc, $l, $r);
@@ -1218,30 +1572,75 @@ The result is in MIX time units.
 
 =item $mix->reset()
 
+Reset machine status to initial state.
+Automatically invoked when a MIX object is created.
+Clear the machine memory and registers to zero.
+Discard all statistics. 
+PC will be cleared to zero, so next time the machine
+will read instruction from location zero.
+
 =item $mix->step()
 
+Execute one instruction. Return 1 if success, otherwise return 0.
+Please check the machine status when this method returns 0.
+
 =item $mix->go()
 
+Simulate MIX go button. Load a card into location 0, and then
+execute instructions from 0 until a halt condition is met, 
+either by error or by the instruction HLT.
+
 =back
 
 =head1 IO DEVICES
 
 General information about MIX io devices.
 
+    Unit number          Device            
+    ===========          ======
+         t               Tape (0 <= t <= 7)
+         d               Disk or drum (8 <= d <= 15)
+         16              Card reader
+         17              Card punch
+         18              Printer
+         19              Typewriter and paper tape
+
+Unit 16,17,18 are oftener used.
+
+Each type of device has its own buffer convention.
+Card reader 's buffer is an array of strings.
+Each string is a card. The maximum length of a card is 80 characters.
+The first item in the array is always the first to be consumed by the machine.
+When MIX executes one C<IN ?,?(16)> instruction, the first card will be read in,
+and that card is also discarded out of the buffer. 
+
+Printer's buffer is for output, it is also array of strings.
+Each string is a page.
+
 =over 4
 
-=item Card reader and punch
+=item $mix->add_device($devnum, $buf)
 
-=item Printer
+Register a device with its buffer.
 
-=item Tape
+=item $mix->get_device_buffer($devnum)
 
-=item Disk and drum
+Return the reference to the device buffer.
 
-=item Paper tape and typewriter
+=item $mix->load_card($loc)    
+
+Load the first card in the card reader buffer into memory, 
+starting from location C<$loc>.
 
 =back
 
+=head1 MIX LOADER
+
+When you press the MIX GO button, MIX firstly loads the first
+card into memory 0000~0015, and then executes from location 0.
+
+Please refer to Section 1.3.1, Exercise 26.
+
 =head1 AUTHOR
 
 Chaoji Li<lichaoji@gmail.com>
@@ -1261,7 +1660,4 @@ Typical usage:
 
     perl mixsim.pl <yourprogram.crd>
 
-Then type 'h' at the command line so you can see a list of commands. You can load a MIX program
-into the machine and see it run.
-
 =cut
